@@ -1,5 +1,19 @@
 <?php
 
+/**
+ +-----------------------------------------------------------------------+
+ | This file is part of the Roundcube Webmail client                     |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ |                                                                       |
+ | Licensed under the GNU General Public License version 3 or            |
+ | any later version with exceptions for skins & plugins.                |
+ | See the README file for a full license statement.                     |
+ +-----------------------------------------------------------------------+
+ | Author: Thomas Bruederli <roundcube@gmail.com>                        |
+ +-----------------------------------------------------------------------+
+*/
+
 if (!class_exists('rcmail_install', false) || !is_object($RCI)) {
     die("Not allowed! Please open installer/index.php instead.");
 }
@@ -11,7 +25,6 @@ $RCI->bool_config_props = array(
   'auto_create_user' => 1,
   'smtp_log' => 1,
   'prefer_html' => 1,
-  'debug_level' => 1,
 );
 
 // allow the current user to get to the next step
@@ -196,21 +209,6 @@ echo $input_ilevel->show($RCI->getprop('identities_level'), 0);
 <legend>Logging & Debugging</legend>
 <dl class="loggingblock">
 
-<dt class="propname">debug_level</dt>
-<dd>
-<?php
-
-$value = $RCI->getprop('debug_level');
-$check_debug = new html_checkbox(array('name' => '_debug_level[]'));
-echo $check_debug->show(($value & 1) ? 1 : 0 , array('value' => 1, 'id' => 'cfgdebug1'));
-echo '<label for="cfgdebug1">Log errors</label><br />';
-
-echo $check_debug->show(($value & 4) ? 4 : 0, array('value' => 4, 'id' => 'cfgdebug4'));
-echo '<label for="cfgdebug4">Print errors (to the browser)</label><br />';
-
-?>
-</dd>
-
 <dt class="propname">log_driver</dt>
 <dd>
 <?php
@@ -476,7 +474,7 @@ $text_smtpport = new html_inputfield(array('name' => '_smtp_port', 'size' => 6,
 echo $text_smtpport->show($RCI->getprop('smtp_port'));
 
 ?>
-<div>SMTP port (default is 25; 465 for SSL; 587 for submission)</div>
+<div>SMTP port (default is 587)</div>
 </dd>
 
 <dt class="propname">smtp_user/smtp_pass</dt>
