@@ -2098,7 +2098,7 @@ class rcmail extends rcube
             }
         }
 
-        $this->output->add_label('selectimage', 'addimage', 'selectmedia', 'addmedia');
+        $this->output->add_label('selectimage', 'addimage', 'selectmedia', 'addmedia', 'close');
         $this->output->set_env('editor_config', $config);
 
         if ($path = $this->config->get('media_browser_css_location', 'program/resources/tinymce/browser.css')) {
@@ -2160,13 +2160,14 @@ class rcmail extends rcube
      * @param string $name       Form object name
      * @param string $action     Form action name
      * @param array  $input_attr File input attributes
+     * @param int    $max_size   Maximum upload size
      *
      * @return string HTML output
      */
-    public function upload_form($attrib, $name, $action, $input_attr = array())
+    public function upload_form($attrib, $name, $action, $input_attr = array(), $max_size = null)
     {
         // Get filesize, enable upload progress bar
-        $max_filesize = $this->upload_init();
+        $max_filesize = $this->upload_init($max_size);
 
         $hint = html::div('hint', $this->gettext(array('name' => 'maxuploadsize', 'vars' => array('size' => $max_filesize))));
 
